C	*************************************************************
C	KRAMERS-KRONIG PROGRAM
C	*************************************************************
	PROGRAM KKTRANS
	CHARACTER*1 GC

C	*************************************************************
C	MAIN SELECTION MENU
C	*************************************************************
11	FORMAT (////////////)
12	FORMAT (16X,' -----------------------------------')
13	FORMAT (16X,' | Kramers-Kronig analysis program |')
14	FORMAT (16X,' |    for transmission spectra     |')
15	FORMAT (16X,' -----------------------------------',/)
16	FORMAT (14X,' V.Ossenkopf, University-Observatory Jena',/////////)
	WRITE (6,11)
	WRITE (6,12)
	WRITE (6,13)
	WRITE (6,14)
	WRITE (6,15)
	WRITE (6,16)
200	CALL READCNST
	CALL PARMMENU
	CALL GRAPH
	CALL SAVECNST
21	FORMAT (///,' Treat another spectrum ? :',$)
22	FORMAT (A1)
23	FORMAT (///////////////////////////////////)
250	WRITE (6,21)
	READ (5,22) GC
	WRITE (6,23)
	IF (GC.EQ.'Y'.OR.GC.EQ.'y') GOTO 200
	END

C	*************************************************************
C	ROUTINE TO READ OPTICAL CONSTANTS OF ONE MEDIUM FROM DISK
C	*************************************************************
	SUBROUTINE READCNST
	INTEGER M,I,IR
	REAL L(256),N(256),K(256),E1(256),E2(256)
	REAL LH(256),NH(256)
	REAL UCST,EKBR
	COMMON /OPTCST/ L,N,K,E1,E2
	COMMON /ANZ/ M
	COMMON /KBR/ EKBR
	DATA UCST /3.97887E-2/

C	*************************************************************
C	READ THE TRANSMISSION SPECTRUM
C	*************************************************************
330	CALL READTABLE(LH,NH,M)
        IF(LH(1).LE.LH(M)) THEN
        DO 300 I=1,M
        L(I)=LH(I)
300	E2(I)=NH(I)
        ELSE
        DO 320 I=1,M
        IR=M+1-I
        L(I)=LH(IR)
320	E2(I)=NH(IR)
        ENDIF
C	*************************************************************
C	USE A CONSTANT DIELECTRIC FUNCTION FOR THE SURROUNDING
C	*************************************************************
11	FORMAT (' Dielectric constant of the embedding medium :',$)
	WRITE (6,11)
	READ (5,*) EKBR
	DO 100 I=1,M
100	E2(I)=E2(I)*L(I)*UCST/SQRT(EKBR)
	RETURN
	END

C	***********************************************************
C	ROUTINES TO READ THE FILE CONTAINING THE EXTINCTION SPECTRUM
C	***********************************************************
	SUBROUTINE READTABLE(ZI,NI,M,NAME)
	INTEGER LOGI,M,I
	REAL ZI(256),NI(256),AA
	CHARACTER*64 LFILE
	CHARACTER*10 NAME

11	FORMAT (' File name of extinction spectrum: ',$)
14      FORMAT (' This file has an illegal extension ! ',/)
30      FORMAT (A64)
100     WRITE (6,11)
        READ (5,30) LFILE
        CALL NAMEEXT(LFILE,NAME,LOGI)
        IF(LOGI.NE.0) GOTO 140
        WRITE (6,14)
        GOTO 100
C	***********************************************************
C	LOADING LINE PARAMETERS FROM FILE CALLED LFILE
C	***********************************************************
140	OPEN(12,ERR=120,FILE=LFILE,STATUS='OLD')
	DO 105 I=1,256
105	READ (12,FMT=*,END=110) ZI(I),NI(I)
110	CLOSE (12)
	M=I-1
12	FORMAT (' Number of lines in the file      : ',I5,/)
	WRITE (6,12) M
C	**********************************************************
C	REDUCTION TO Q/A [CM^-1] DEPENDING ON LOGI
C	**********************************************************
16	FORMAT (' Radius of the grains [cm]        : ',$)
17	FORMAT (' Density of the grains [g/cm^3]   : ',$)
        IF (ABS(LOGI).EQ.1) THEN
	WRITE (6,16)
	READ (5,*) AA
	DO 190 I=1,M
190	NI(I)=NI(I)/AA
	ELSE IF (ABS(LOGI).EQ.2) THEN
	DO 150 I=1,M
150	NI(I)=1E4*NI(I)
	ELSE IF (ABS(LOGI).EQ.3) THEN
	DO 160 I=1,M
160	NI(I)=1.33333*NI(I)
	ELSE
	WRITE (6,17)
	READ (5,*) AA
	DO 170 I=1,M
170	NI(I)=1.33333*AA*NI(I)
	ENDIF
	IF (LOGI.LT.0) THEN
        DO 180 I=1,M
180     ZI(I)=1.0/ZI(I)
	ENDIF
	RETURN
13	FORMAT (' There is no access to file       : ',A32,/)
120	WRITE (6,13) LFILE
	GOTO 100
	END

C       *************************************************************
C       CHECK IF THE FILE IS A ".LQR", ".LCV", OR ".LKP"-FILE
C       RECOGNITION OF THE TITLE OF THE SPECTRUM
C       *************************************************************
        SUBROUTINE NAMEEXT(FILEN,NAME,LZ)
        INTEGER EXA,NME,LZ,LZH
        CHARACTER*10 NAME
        CHARACTER*64 FILEN,HNAME,HFNAME
        CHARACTER*1 EXT1
	CHARACTER*2 EXT2
	CHARACTER*3 EXTENS

        HNAME=FILEN
200     EXA=INDEX(HNAME,'/')+1
        IF (EXA.GT.1) THEN
        HNAME=HNAME(EXA:)
        GOTO 200
        ELSE
        HFNAME=HNAME
        ENDIF

        HNAME=HFNAME
        NME=0
100     EXA=INDEX(HNAME,'.')+1
        IF (EXA.GT.1) THEN
        HNAME=HNAME(EXA:)
        NME=NME+EXA-1
        GOTO 100
        ELSE
        EXTENS=HNAME
	EXT1=EXTENS(:1)
	EXT2=EXTENS(2:)
        NAME=HFNAME(:MAX0(0,NME-1))
        ENDIF

        LZ=0
        IF(EXT2.EQ.'QA'.OR.EXT2.EQ.'qa') LZ=1
        IF(EXT2.EQ.'QR'.OR.EXT2.EQ.'qr') LZ=2
        IF(EXT2.EQ.'CV'.OR.EXT2.EQ.'cv') LZ=3
        IF(EXT2.EQ.'KP'.OR.EXT2.EQ.'kp') LZ=4
        LZH=0
        IF(EXT1.EQ.'L'.OR.EXT1.EQ.'l') LZH=1
        IF(EXT1.EQ.'Z'.OR.EXT1.EQ.'z') LZH=-1
	LZ=LZ*LZH
        RETURN
        END

C	*************************************************************
C	ROUTINE TO SAVE A CALCULATED SPECTRUM OF OPTICAL CONSTANTS
C	ON DISK
C	*************************************************************
	SUBROUTINE SAVECNST
	INTEGER M
	REAL L(256),N(256),K(256),E1(256),E2(256)
	CHARACTER*10 DUMMY
	COMMON /OPTCST/ L,N,K,E1,E2
	COMMON /ANZ/ M

	CALL SAVEFILE(L,N,K,M,DUMMY)
	RETURN
	END

C	*************************************************************
C	SUBROUTINE TO SHOW OPTICAL CONSTANTS GRAPHICALLY
C	*************************************************************
	SUBROUTINE GRAPH
	INTEGER I,EK,NIN
	INTEGER M,NN(4)
	REAL L(256),CST(256,4)
	REAL XF(256,4),YF(256,4)
	CHARACTER*40 TITLE
	CHARACTER*2 ORDTIT
	CHARACTER*10 ABSTIT
	COMMON /OPTCST/ L,CST
	COMMON /ANZ/ M

27	FORMAT (/////////////////////////////////,
     % ' Error occured in graphics mode - try to continue !')
C	*************************************************************
C	SHOW SELECTED CONSTANT(LAMBDA)
C	*************************************************************
	NN(1)=M
	NIN=1
	ABSTIT='wavelength'
	DO 200 I=1,M
	XF(I,1)=L(I)
200	YF(I,1)=CST(I,1)
	TITLE='Calculated refractive index'
	ORDTIT=' n'
	CALL PLOT(XF,YF,NN,NIN,ORDTIT,ABSTIT,TITLE,EK)
	IF(EK.EQ.0) GOTO 220
	WRITE (6,27)
220	DO 230 I=1,M
230	YF(I,1)=CST(I,2)
	TITLE='Calculated absorption coefficient'
	ORDTIT=' k'
	CALL PLOT(XF,YF,NN,NIN,ORDTIT,ABSTIT,TITLE,EK)
	IF(EK.EQ.0) GOTO 240
	WRITE (6,27)
	CALL SMLPAUSE
240	RETURN
	END

C	*************************************************************
C	SUBROUTINE TO SHOW EXTRAPOLATED OPTICAL CONSTANTS 
C	*************************************************************
	SUBROUTINE EXGRAPH(KOMP,LPARM,RPARM)
	INTEGER KOMP,EK
	INTEGER M,NN(4),I0,I
	REAL LPARM(6),RPARM(6)
	REAL L(256),CST(256,4)
	REAL XF(256,4),YF(256,4)
	REAL LFUNCT,RFUNCT
	CHARACTER*40 TITLE
	CHARACTER*2 ORDTIT
	CHARACTER*10 ABSTIT
	COMMON /OPTCST/ L,CST
	COMMON /ANZ/ M

C       *************************************************************
C       FIRST GRAPH - ORIGINAL DATA
C       *************************************************************
        DO 100 I=1,M
        XF(I,1)=L(I)
100     YF(I,1)=CST(I,KOMP)
        NN(1)=M
C	*************************************************************
C	CALCULATE EXTRAPOLATED POINTS
C	*************************************************************
	I0=0
	IF (LPARM(1).GT.0.0) THEN
	DO 200 I=1,9
	XF(I,2)=L(1)/10.0*I
200	YF(I,2)=LFUNCT(XF(I,2),LPARM)
	I0=I0+9
	ENDIF
	DO 300 I=1,M
	XF(I+I0,2)=L(I)
300	YF(I+I0,2)=CST(I,KOMP)
	I0=I0+M
	IF (RPARM(1).GT.0.0) THEN
	DO 400 I=1,9
	XF(I0+I,2)=L(M)*(I+1)
400	YF(I0+I,2)=RFUNCT(XF(I0+I,2),RPARM)
	I0=I0+9
	ENDIF
	NN(2)=I0
27      FORMAT (///////////////////////////////,
     % ' Error occured in graphics mode - try to continue !')
C	*************************************************************
C	SHOW SELECTED CONSTANT(LAMBDA)
C	*************************************************************
        TITLE='Behaviour of the chosen extrapolation'
	ABSTIT='wavelength'
	ORDTIT=' f'
	CALL PLOT(XF,YF,NN,2,ORDTIT,ABSTIT,TITLE,EK)
	IF(EK.EQ.0) GOTO 220
	WRITE (6,27)
	CALL SMLPAUSE
220	RETURN
	END

C	***********************************************************
C	KRAMERS-KRONIG ANALYSIS ROUTINE
C	***********************************************************
	SUBROUTINE KKCALC(KOMP,LPARM,RPARM)
	INTEGER KOMP,KOMPNEU
	REAL RPARM(6),LPARM(6)
	REAL L(256),CST(256,4)
	REAL X(256),F(256),D(256)
	REAL A1(255),A2(255),A3(255),A4(255)
	REAL ERR,PI,EKBR,E1,E2,HLP
	REAL LMIN,LMAX
	REAL*8 LC,LCST,L1,L2
	REAL*8 INT,CONTROL,CUBINT,INFINT,LINT,RINT
	INTEGER M,N,I,J
	LOGICAL LENDE,RENDE
	COMMON /OPTCST/ L,CST
	COMMON /ANZ/ M
	COMMON /KBR/ EKBR
	DATA ERR /1E-4/
	DATA PI /3.1415927/

C	**********************************************************
C	CHECK
C	**********************************************************
	IF (LPARM(1).GE.1.0.AND.RPARM(1).GE.1.0) GOTO 105
11	FORMAT (' Extrapolation not initialized !')
	WRITE (6,11)
	CALL SMLPAUSE
	RETURN
105	KOMPNEU=KOMP+(-1)**(KOMP+1)
C	**********************************************************
C	INITIALIZE CUBIC SPLINE PARAMETERS
C	**********************************************************
	N=M
	LMIN=L(1)
	LMAX=L(N)
	DO 100 I=1,N
	X(I)=L(I)
100	F(I)=CST(I,KOMP)
	CALL SPLINE(X,F,N,D)
	DO 110 I=1,N-1
	A1(I)=((D(I)+D(I+1))*(X(I+1)-X(I))-2.0*(F(I+1)-F(I)))/
     %  (X(I+1)-X(I))**3
	A2(I)=(D(I+1)-D(I)-3.0*A1(I)*(X(I+1)**2-X(I)**2))/
     %  (2.0*(X(I+1)-X(I)))
	A3(I)=D(I)-3.0*A1(I)*X(I)**2-2.0*A2(I)*X(I)
	A4(I)=F(I)-A1(I)*X(I)**3-A2(I)*X(I)**2-A3(I)*X(I)
110	CONTINUE
12	FORMAT (////////,' Please, wait some seconds - Kramers-Kronig inte
     %gration is running.')
	WRITE (6,12) 
C	**********************************************************
C	CONTROL INTEGRATION RELATIVE TO AN INFINITE WAVELENGTH TO
C	CHECK WHETHER THE GIVEN ABSORPTION EFFICIENCIES MAY BE IN
C	ACCORDANCE WITH THE RESTRICTION EPSILON(LAMDA=INF)>0
C	**********************************************************
	CONTROL=0.0D0
	DO 600 I=1,N-1
	L1=DBLE(L(I))
	L2=DBLE(L(I+1))
600	CONTROL=CONTROL+INFINT(L1,L2,A1(I),A2(I),A3(I),A4(I))
	CONTROL=CONTROL*2.0D0/PI
31	FORMAT (///,' Contribution to the static function : ',F10.4)
32	FORMAT (/,' The given absorption efficiencies are to high and the
     %refore unphysical !')
	WRITE (6,31) CONTROL
	IF (CONTROL.GT.1D0) THEN
	WRITE (6,32)
	CALL SMLPAUSE
	ENDIF
C	**********************************************************
C	INTEGRATION
C	**********************************************************
	DO 200 I=1,N
	LC=DBLE(X(I))
C	**********************************************************
C	DETERMINATION OF THE CONSTANT REGION AROUND THE SINGULARITY
C	**********************************************************
	IF (I.LT.N) THEN
	LCST=DBLE(X(I+1)-X(I))
	ELSE 
	LCST=DBLE(X(N)-X(N-1))
	ENDIF
	LCST=DMIN1(0.1D0*LCST,DBLE(ERR*ABS(F(I)/D(I))),ERR*LC)
	LENDE=.FALSE.
	RENDE=.FALSE.
	INT=0.0D0
C	BEGINN
C	LINKS
	IF (I.EQ.1) THEN
	INT=INT+LINT(KOMP,LC,LC-LCST,LPARM)
	LENDE=.TRUE.
	ELSE
	INT=INT+CUBINT(KOMP,LC,DBLE(X(I-1)),LC-LCST,A1(I-1),A2(I-1),
     %  A3(I-1),A4(I-1))
	ENDIF
C	RECHTS
	IF (I.EQ.N) THEN
	INT=INT+RINT(KOMP,LC,LC+LCST,RPARM)
	RENDE=.TRUE.
	ELSE
	INT=INT+CUBINT(KOMP,LC,LC+LCST,DBLE(X(I+1)),A1(I),A2(I),A3(I),
     %  A4(I))
	ENDIF
	J=0
C	WEITER
210	J=J+1
C	LINKS
	IF (LENDE) GOTO 220
	IF (I-J.EQ.1) THEN
	INT=INT+LINT(KOMP,LC,DBLE(LMIN),LPARM)
	LENDE=.TRUE.
	ELSE
	INT=INT+CUBINT(KOMP,LC,DBLE(X(I-J-1)),DBLE(X(I-J)),A1(I-J-1),
     %  A2(I-J-1),A3(I-J-1),A4(I-J-1))
	ENDIF
220	CONTINUE
	IF (RENDE) GOTO 230
	IF (I+J.EQ.N) THEN
	INT=INT+RINT(KOMP,LC,DBLE(LMAX),RPARM)
	RENDE=.TRUE.
	ELSE
	INT=INT+CUBINT(KOMP,LC,DBLE(X(I+J)),DBLE(X(I+J+1)),A1(I+J),
     %  A2(I+J),A3(I+J),A4(I+J))
	ENDIF
230	CONTINUE
	IF (.NOT.(LENDE.AND.RENDE)) GOTO 210
	INT=INT*2.0D0/PI
200	CST(I,KOMPNEU)=INT
C	*************************************************************
C	CALCULATION OF THE OTHER PAIR OF CONSTANTS
C	*************************************************************
	DO 320 I=1,M
	HLP=CST(I,3)**2+CST(I,4)**2
	E1=(1.0+CST(I,3)-2.0*HLP)/(1.0-2.0*CST(I,3)+HLP)*EKBR
	E2=3.0*CST(I,4)/(1.0-2.0*CST(I,3)+HLP)*EKBR
	HLP=SQRT(E1**2+E2**2)
	CST(I,1)=SQRT(0.5*(HLP+E1))
320	CST(I,2)=SQRT(0.5*(HLP-E1))
	RETURN
	END

C	*************************************************************
C	INTEGRALS INSIDE THE GIVEN WAVELENGTH RANGE RELATIVE TO AN
C	INFINITE WAVELENGTH
C	*************************************************************
	REAL*8 FUNCTION INFINT(X1,X2,A1,A2,A3,A4)
	REAL*8 X1,X2
	REAL A1,A2,A3,A4

	INFINT=A1*(X2**3-X1**3)/3D0+A2*(X2**2-X1**2)*0.5D0+A3*(X2-X1)
     % +A4*DLOG(X2/X1)
	RETURN
	END

